/***************************************************************************
 *   Copyright (C) 2004 by Cyril Bosselut                                  *
 *   bosselut@b1project.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ktagmainwin.h"


KTagMainWin::KTagMainWin( QWidget* parent, const char* name, WFlags fl ) : DCOPObject( "KTag" ), KMainWindow( parent, name, fl ){
	mConfig = kapp->config();
	setupActions();
	setAcceptDrops(TRUE);
	readProperties();
	tabWidget = new KTabWidget( this, "tabWidget" );
	tabWidget->setEnabled( true );
	tabWidget->setHoverCloseButton( TRUE );
	tabWidget->setAutoMask( TRUE );
	tabWidget->setAcceptDrops(TRUE);
	setCentralWidget(tabWidget);
	connect(kapp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
	connect(tabWidget, SIGNAL(closeRequest(QWidget *)), this, SLOT(closeTab(QWidget *)));
	connect(tabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(updateAlbumMenu(QWidget *)));
	connect(tabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(updateFileMenu(QWidget *)));
}

KTagMainWin::~KTagMainWin(){
}

void KTagMainWin::setupActions(){
	KStdAction::save(this, SLOT(saveFile()), actionCollection());
	KStdAction::preferences(this, SLOT(config()), actionCollection());
	KStdAction::open(this, SLOT(chooseFile()), actionCollection());
	recent = KStdAction::openRecent(this, SLOT(openFile(const KURL&)), actionCollection());
	mConfig->setGroup("KFileDialog Settings");
	recent->loadEntries(mConfig);
	(void) new KAction(tr2i18n("Save &All"), 0, this, SLOT(saveAll()), actionCollection(), "saveAll");
	(void) new KAction(tr2i18n("Download cover"), 0, this, SLOT(getAlbumCover()), actionCollection(), "getCover");
	createGUI();
}

void KTagMainWin::saveFile(){
	kdDebug("MainWin") << "Saving current file" << endl;
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	mKtag->writeTags(false);
	kdDebug("MainWin") << mKtag->mAudioFile << endl;
}

void KTagMainWin::saveAll(){
	if(tabWidget->count() > 0){
		KProgressDialog *p = new KProgressDialog(this, "progress", tr2i18n("Writing tags"), tr2i18n("Saving all files"), true);
		p->showCancelButton(false);
		p->progressBar()->setTotalSteps(tabWidget->count());
		kdDebug("MainWin") << "Saving all tabs " << endl;
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			if(mKtag->modified){
				mKtag->writeTags(true);
			}
			p->progressBar()->setProgress(i+1);
		}
	}
}

void KTagMainWin::closeEvent(QCloseEvent *ce){
	if(tabWidget->count() > 0){
		kdDebug("MainWin") << "Closing tabs " << endl;
		while(tabWidget->count() > 0){
			closeTab(tabWidget->page(0));
		}
	}
	ce->accept();
}

bool KTagMainWin::chooseFile(){
	KURL url;
	url = KFileDialog::getOpenURL(QString::null, "*.mp3 *.ogg", 0, "Open audio file");
	if( url.isLocalFile() ){
		openFile(url);
		return true;
	}
	return false;
}

void KTagMainWin::openFile(const KURL &url){
	Ktag* mKtag;
	KIconLoader* ic = new KIconLoader("ktag");
	mKtag = new Ktag(this, url.path());
	if(mKtag->setFile(url.path())){
		kdDebug("MainWin") << "Adding new tab" << endl;
		tabWidget->insertTab( mKtag, "" );
		tabWidget->setTabIconSet( mKtag, ic->loadIconSet( "sound_mp3", KIcon::Toolbar ) );
		updateTabLabel(mKtag, mKtag->mAudioFile);
		connect(mKtag, SIGNAL(tabClosed(QWidget *)), this, SLOT(closeTab(QWidget *)));
		connect(mKtag, SIGNAL(modifiedTag(QWidget *, bool)), this, SLOT(updateTabLabel(QWidget *, bool)));
		connect(mKtag, SIGNAL(modifiedTag(QWidget *, bool)), this, SLOT(updateAlbumMenu(QWidget *)));
		connect(mKtag, SIGNAL(modifiedTag(QWidget *, bool)), this, SLOT(updateFileMenu(QWidget *)));
		connect(mKtag, SIGNAL(fileChanged(QWidget *, const QString&)), this, SLOT(updateTabLabel(QWidget *, const QString&)));
		connect(mKtag, SIGNAL(receivedDropEvent(QDropEvent *)), this, SLOT(dropEvent(QDropEvent *)));
		tabWidget->showPage(mKtag);
		mKtag->setAcceptDrops(TRUE);
		recent->saveEntries(mConfig);
	}
	mKtag = 0;
	if(tabWidget->count() == 0){
		kdDebug("MainWin") << "no more tabs, exit" << endl;
		exit(0);
	}
}

void KTagMainWin::openFile(const QString& fp){
	kdDebug("MainWin") << fp << endl;
	KURL url;
	if(fp.contains("%")){
		url.setEncodedPath(QString(fp).remove("file:"));
	}
	else{
		url.setPath(fp);
	}
	openFile(url);
}


void KTagMainWin::closeTab( QWidget* w ){
	kdDebug("MainWin") << "Closing tab " << w->name() << endl;
	w->close();
	tabWidget->removePage(w);
	if(tabWidget->count() == 0){
		qApp->quit();
	}
}

void KTagMainWin::updateTabLabel(QWidget*w, bool b){
	Ktag* mKtag = static_cast<Ktag*>(w);
	if(b && QFileInfo(mKtag->mAudioFile).isWritable()){
		tabWidget->setTabColor(w, QColor("blue"));
		return;
	}
	if(!QFileInfo(mKtag->mAudioFile).isWritable()){
		tabWidget->setTabColor(w, QColor("grey"));
		return;
	}
	tabWidget->setTabColor(w, QColor("black"));
}

void KTagMainWin::updateTabLabel(QWidget*w, const QString& str){
	KURL url(str);
	QString filename = url.fileName(true);
	tabWidget->setTabLabel(w, filename);
	recent->addURL(url);
	recent->saveEntries(kapp->config());
}

void KTagMainWin::dragMoveEvent( QDragMoveEvent *e){
	e->accept(false);
}

void KTagMainWin::dropEvent( QDropEvent *e){
	kdDebug("Ktag") << "File dropped" << endl;
	if(e->provides("text/uri-list")){
		e->accept( true );
		e->encodedData("text/uri-list");
		QStrList lst;
		QString filename;
		QUriDrag::decode(e, lst);
		for ( uint i = 0; i < lst.count(); ++i ) {
			openFile(QString(QDir::convertSeparators(QUriDrag::uriToLocalFile(lst.at(i))).utf8()));
		}
	}
	else{
		e->ignore();
		kdDebug("Ktag") << "Droped file rejected: " << e->format() << endl;
	}
}

void KTagMainWin::config(){
	//An instance of your dialog could be already created and could be cached,
	//in which case you want to display the cached dialog instead of creating
	//another one
	if ( KConfigDialog::showDialog( "configdlg" ) )
		return;

	//KConfigDialog didn't find an instance of this dialog, so lets create it :
	KConfigDialog* dialog = new KConfigDialog( this, "configdlg", Prefs::self() );
	patternConfig* patDlg = new patternConfig( 0, "dlPatterns" );
	confSavePatDlg* saveDlg = new confSavePatDlg( 0, "savePatterns" );

	dialog->addPage( patDlg, i18n("Cover patterns"), "package_network" );
	dialog->addPage( saveDlg, i18n("File patterns"), "package_editors" );

	patDlg->addPatternList(mPatterns);
	saveDlg->addPatternList(mSavePatterns);
	//User edited the configuration - update your local copies of the
	//configuration data
	//connect( dialog, SIGNAL(settingsChanged()), this, SLOT(saveProperties()) );

	//dialog->show();
	if(dialog->exec()){
		kdDebug("Ktag") << "Save config" << endl;
		mPatterns.clear();
		for(int i = 0; i <= patDlg->kcfg_dlPatterns->childCount()-1; i++){
			mPatterns << patDlg->kcfg_dlPatterns->itemAtIndex(i)->text(0);
		}
		mSavePatterns.clear();
		for(int i = 0; i <= saveDlg->kcfg_savePatterns->childCount()-1; i++){
			mSavePatterns << saveDlg->kcfg_savePatterns->itemAtIndex(i)->text(0);
		}
		saveProperties();
		readProperties();
	}
	else{
		kdDebug("Ktag") << "Do not save config" << endl;
	}
}

void KTagMainWin::updateStatus(QWidget* w,const QString& str){
	tabWidget->setTabToolTip(w, str);
}

void KTagMainWin::saveProperties(){
	mConfig->setGroup("ktag");
	mConfig->writeEntry("patterns", mPatterns.join(","));
	mConfig->writeEntry("svpatterns", mSavePatterns.join(","));
	mConfig->sync();
}

void KTagMainWin::readProperties(){
	kdDebug("Ktag") << "Read config" << endl;
	mConfig->setGroup("ktag");
	mPatterns = mConfig->readListEntry("patterns", ',');
	mSavePatterns = mConfig->readListEntry("svpatterns", ',');
	QStringList::ConstIterator it = mSavePatterns.begin();
	QStringList::ConstIterator end = mSavePatterns.end();
	QPopupMenu *saveAsPopup = static_cast<QPopupMenu*>(factory()->container("saveas",this));
	QPopupMenu *saveTrckAsPopup = static_cast<QPopupMenu*>(factory()->container("saveTrckAs",this));
	QPopupMenu *renamePopup = static_cast<QPopupMenu*>(factory()->container("rename",this));
	QPopupMenu *renameTrckAsPopup = static_cast<QPopupMenu*>(factory()->container("renameTrckAs",this));
	saveAsPopup->clear();
	saveTrckAsPopup->clear();
	renamePopup->clear();
	renameTrckAsPopup->clear();
	while( it != end ){
		saveAsPopup->insertItem( *it,  this, SLOT(saveAs(int)));
		saveTrckAsPopup->insertItem( *it,  this, SLOT(saveAllAs(int)));
		renamePopup->insertItem( *it,  this, SLOT(rename(int)));
		renameTrckAsPopup->insertItem( *it,  this, SLOT(renameAll(int)));
		it++;
	}
}

void KTagMainWin::saveAs(int i){
	QPopupMenu *saveAsPopup = static_cast<QPopupMenu*>(factory()->container("saveas",this));
	saveAs(saveAsPopup->text(i));
}

void KTagMainWin::saveAllAs(int i){
	QPopupMenu *saveTrckAsPopup = static_cast<QPopupMenu*>(factory()->container("saveTrckAs",this));
	saveAllAs(saveTrckAsPopup->text(i));
}

void KTagMainWin::rename(int i){
	QPopupMenu *renamePopup = static_cast<QPopupMenu*>(factory()->container("rename",this));
	rename(renamePopup->text(i));
}

void KTagMainWin::renameAll(int i){
	QPopupMenu *renameTrckAsPopup = static_cast<QPopupMenu*>(factory()->container("renameTrckAs",this));
	renameAll(renameTrckAsPopup->text(i));
}

void KTagMainWin::saveAs(const QString& p){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	mKtag->saveAs(p);
}

void KTagMainWin::saveAllAs(const QString& p){
	if(tabWidget->count() > 0){
		KProgressDialog *progress = new KProgressDialog(this, "progress", tr2i18n("Copying files"), tr2i18n("Copying files"), true);
		progress->showCancelButton(false);
		progress->progressBar()->setTotalSteps(tabWidget->count());
		kdDebug("MainWin") << "Copying files" << endl;
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->rename(p);
			progress->progressBar()->setProgress(i+1);
		}
	}
}

void KTagMainWin::rename(const QString& p){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	mKtag->rename(p);
}

void KTagMainWin::renameAll(const QString& p){
	if(tabWidget->count() > 0){
		KProgressDialog *progress = new KProgressDialog(this, "progress", tr2i18n("Renaming files"), tr2i18n("Renaming files"), true);
		progress->showCancelButton(false);
		progress->progressBar()->setTotalSteps(tabWidget->count());
		kdDebug("MainWin") << "Renaming files" << endl;
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->rename(p);
			progress->progressBar()->setProgress(i+1);
		}
	}
}

void KTagMainWin::updateAlbumMenu(QWidget *w){
	Ktag* mKtag = static_cast<Ktag*>(w);
	QPopupMenu *artistPopup = static_cast<QPopupMenu*>(factory()->container("setArtist",this));
	QPopupMenu *albumPopup = static_cast<QPopupMenu*>(factory()->container("setAlbum",this));
	QPopupMenu *yearPopup = static_cast<QPopupMenu*>(factory()->container("setYear",this));
	QPopupMenu *genrePopup = static_cast<QPopupMenu*>(factory()->container("setGenre",this));
	artistPopup->clear();
	albumPopup->clear();
	yearPopup->clear();
	genrePopup->clear();
	if(!mKtag->mArtist->currentText().isEmpty()){
		artistPopup->insertItem( mKtag->mArtist->currentText(),  this, SLOT(setArtist()));
	}
	if(!mKtag->mAlbum->currentText().isEmpty()){
		albumPopup->insertItem( mKtag->mAlbum->currentText(),  this, SLOT(setAlbum()));
	}
	if(mKtag->mYear->value() != 0){
		yearPopup->insertItem( QString::number(mKtag->mYear->value()),  this, SLOT(setYear()));
	}
	if(!mKtag->mGenre->currentText().isEmpty()){
		genrePopup->insertItem( mKtag->mGenre->currentText(),  this, SLOT(setGenre()));
	}
}

void KTagMainWin::updateFileMenu(QWidget *w){
	Ktag* mKtag = static_cast<Ktag*>(w);
	KAction *save = actionCollection()->action(KStdAction::stdName(KStdAction::Save));
	save->setEnabled(mKtag->modified);
}

void KTagMainWin::setArtist(){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	setArtist(mKtag->mArtist->currentText());
}

void KTagMainWin::setArtist(const QString& p){
	if(tabWidget->count() > 0){
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->mArtist->setCurrentText(p);
		}
	}
}

void KTagMainWin::setAlbum(){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	setAlbum(mKtag->mAlbum->currentText());
}

void KTagMainWin::setAlbum(const QString& p){
	if(tabWidget->count() > 0){
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->mAlbum->setCurrentText(p);
		}
	}
}

void KTagMainWin::setGenre(){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	setGenre(mKtag->mGenre->currentText());
}

void KTagMainWin::setGenre(const QString& p){
	if(tabWidget->count() > 0){
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->mGenre->setCurrentText(p);
		}
	}
}

void KTagMainWin::setYear(){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	setYear(mKtag->mYear->value());
}

void KTagMainWin::setYear(const int y){
	if(tabWidget->count() > 0){
		for(int i = 0; i <= tabWidget->count()-1; i++){
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->page(i));
			mKtag->mYear->setValue(y);
		}
	}
}

void KTagMainWin::getAlbumCover(){
	Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
	getAlbumCover(mKtag->mArtist->currentText(), mKtag->mAlbum->currentText(), KURL(mKtag->mAudioFile).directory(true));
}

void KTagMainWin::getAlbumCover(const QString &artist, const QString &album, const QString &folder){
	dlCover* dl = new dlCover();
	Tags *tags = new Tags();
	QString ar = artist.lower().replace(" ", "_").replace("'", "_").replace(",", "");
	QString al = album.lower().replace(" ", "_").replace("'", "_").replace(",", "");
	QString l = ar.left(1).lower();
	QString Ar = tags->upFirst(ar).replace(" ","_");
	QString Al = tags->upFirst(al).replace(" ","_");
	QString L = l.upper();
	QStringList::Iterator it = mPatterns.begin();
	QStringList::Iterator end = mPatterns.end();
	QPtrList<Cover> coverList;
	coverList.clear();
	int n = 0;
	kdDebug("Tags") << ar << " | "<< al << endl;
	while( it!= end ){
		Cover* dc = new Cover();
		QString dcmime = QString::null;
		QString str = *it;
		KURL url(str.replace("%l", l).replace("%L", L).replace("%artist", ar).replace("%album", al).replace("%Artist", Ar).replace("%Album", Al));
		kdDebug("Tags") << url.url() << endl;
		dcmime = KIO::NetAccess::mimetype( url, NULL );
		if( dcmime == "image/jpeg" ){
			dc->pixmap = tags->dlCover(url);
			dc->url = url.url();
			dc->width = dc->pixmap->width();
			dc->height = dc->pixmap->height();
			dc->size = (KFileItem::KFileItem(url, dcmime, KFileItem::Unknown)).size();
		}
		if(!dc->pixmap->isNull()){
			KListViewItem* i = new KListViewItem(dl->coverList);
			i->setPixmap(0, dc->pixmap->convertToImage ().smoothScale(128, 128, QImage::ScaleMin));
			i->setText(1,dc->url);
			if(dc->size > 0){
				i->setText(0, i18n("Width: %1px\nHeight: %2px\nSize: %3Ko").arg(dc->width).arg(dc->height).arg(dc->size) );
			}
			else{
				i->setText(0, i18n("Width: %1px\nHeight: %2px\nSize: unknown").arg(dc->width).arg(dc->height) );
			}
			i->setText(2, QString::number(n));
			n++;
			dl->coverList->hideColumn(2);
			coverList.append(dc);
		}
		it++;
	}
	if(dl->coverList->childCount() != 0){
		if(dl->exec() && !dl->coverList->selectedItems().isEmpty()){
			QString nfileName;
			nfileName = KFileDialog::getSaveFileName(folder + "/cover.png", "*.png", 0, "Save Cover");
			if(nfileName != ""){
				(coverList.at(dl->coverList->selectedItem()->text(2).toInt())->pixmap)->save(nfileName, "PNG", 0);
			}
			Ktag* mKtag = static_cast<Ktag*>(tabWidget->currentPage());
			mKtag->mCoverInfo->setEnabled(true);
			mKtag->mCover->setEnabled(true);
			mKtag->mCover->setPixmap(folder + "/cover.png");
			mKtag->mCoverInfo->setText(tr2i18n("<b>Local file:</b><br>") + folder + "/cover.png");

		}
		return;
	}
	KMessageBox::sorry(this, tr2i18n("No cover found for album \"%1\" by \"%2\"").arg(Al, Ar), QString::null, KMessageBox::AllowLink);
}

#include "ktagmainwin.moc"
