/***************************************************************************
 *   Copyright (C) 2004 by Cyril Bosselut                                  *
 *   bosselut@b1project.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KTAG_H_
#define _KTAG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qtooltip.h>
#include <qlabel.h>
#include <qmovie.h>
#include <qpopupmenu.h>
#include <qfileinfo.h>

#include <kmainwindow.h>
#include <klocale.h>
#include <kactivelabel.h>
#include <kcombobox.h>
#include <knuminput.h>
#include <kurlrequester.h>
#include <klineedit.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kprogress.h>
#include <klistview.h>
#include <kio/jobclasses.h>
#include "kio/job.h"
#include <kiconloader.h>

#include "libktagui/ktagdlg.h"
#include "libktagui/dlcover.h"
#include "libtags/tags.h"
#if HAVE_MUSICBRAINZ
#include "musicbrainz/musicbrainzquery.h"
#endif // HAVE_MUSICBRAINZ

/**
 * @short Application Main Window
 * @author Cyril Bosselut <bosselut@b1project.com>
 * @version 0.1
 */

class Ktag : public ktagDlg
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
	Ktag( QWidget* parent = 0, const char* name = 0, WFlags fl = WType_TopLevel );

    /**
     * Default Destructor
     */
	virtual ~Ktag();
	bool setFile( const QString &fp );
	QString mAudioFile;
	bool modified;

private:
#if HAVE_MUSICBRAINZ
	QPopupMenu *mBrainzPopup;
#endif // HAVE_MUSICBRAINZ
	QString newFile;
	Tags *tags;

protected:
	void dropEvent(QDropEvent *);
	void dragMoveEvent( QDragMoveEvent *);
	void closeEvent(QCloseEvent *);

private slots:
	void saveCover();
	bool readTags();
	void isModified(const QString&);
	void isModified(int);
	void downloadCover();
	void slotJobResult(KIO::Job *job);
#if HAVE_MUSICBRAINZ
	void mbSearch();
	void updateLists( const MusicBrainzQuery::TrackList& res );
	void updateMbList(QStringList&);
	void setFromMb(int);
	void mbPopup(QPoint);
#endif // HAVE_MUSICBRAINZ

public slots:
	void writeTags();
	void writeTags(bool);
	void saveAs(const QString &p);
	void rename(const QString &p);

signals:
	void tabClosed( QWidget* );
	void modifiedTag( QWidget*, bool );
	void receivedDropEvent (QDropEvent *);
#if HAVE_MUSICBRAINZ
	void mbStart(QStringList& );
	void mbFound(QStringList& );
#endif // HAVE_MUSICBRAINZ
	void fileChanged( QWidget*,const QString& );

};

#endif // _KTAG_H_
