//
//
// C++ Interface: Tags
//
// Description:
//
//
// Author: Bosselut Cyril <bosselut@b1project.com>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef TAGS_H
#define TAGS_H
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <id3/tag.h>
#include <id3/misc_support.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include <stdlib.h>
#include <stdio.h>

#include <qurl.h>
#include <qfile.h>
#include <qimage.h>
#include <qvariant.h>
#include <qbuffer.h>

#include <kstandarddirs.h>
#include <kio/netaccess.h>
#include <kio/jobclasses.h>
#include <kmimemagic.h>
#include <kfilemetainfo.h>
#include <klibloader.h>
#include <kdebug.h>
#include <kfileitem.h>

/**
* @author Bosselut Cyril <bosselut@b1project.com>
* @version 0.23
*/
class Cover{
public:
	Cover();
	~Cover();

	QPixmap* pixmap;
	QString url;
	int width;
	int height;
	int size;
	void empty(void);
};

class Tags
{
public:
	Tags( const QString & fn );
	Tags(  );

	~Tags();

	void setInfos(const QString & tr, const QString & ar, const QString & ti, const QString & al, const QString & ge, const QString & ye, const QString & ct, const QString & co = NULL);
	void ReadId3();
	void WriteId3();
	void ReadVorbis();
	void WriteVorbis();
	/**
	* audio file
	*/
	QString file;
	/**
	* mimetype of audio file
	*/
	QString mime;
	/**
	* Return true if file is an mp3
	*/
	bool isMP3();
	/**
	* Return true if file is an ogg
	*/
	bool isOGG();
	/**
	* Temporary cover file
	*/
	QString fp;
	/**
	* artist + title + album or xmms output title
	*/
	QString title;
	/**
	* artist
	*/
	QString artist;
	/**
	* album
	*/
	QString album;
	/**
	* song title
	*/
	QString stitle;
	/**
	* track number
	*/
	QString track;
	/**
	* year
	*/
	QString year;
	/**
	* genre
	*/
	QString genre;
	/**
	* comments
	*/
	QString comment;
	/**
	* last position in playlist
	*/
	int current;
	/**
	* bitrate in bps
	*/
	int rate;
	/**
	* frequency in hz
	*/
	double freq;
	/**
	* number of channel (1 = mono, 2 = stereo)
	*/
	int channel;
	/**
	* pointer on id3 tag
	*/
	ID3_Tag* myTag;
	/**
	* album cover
	*/
	QImage cover;
	/**
	* location of new cover
	*/
	QString ncover;
	/**
	* true if id3 contains cover
	*/
	bool haveCover;
	/**
	* get search cover on Darktown
	*/
	Cover* netCover(const QString& pattern);
	/**
	* download cover
	*/
	QPixmap* dlCover(KURL url);
	/**
	* Case upper the first letter of each words
	*/
	QString upFirst(const QString & str);
	static const int NB_GENRES;

};

// this part comes from KTagit http://ktagit.sourceforge.net/
static const char* sorted_genres[] = {
	"", "ACapella", "Acid", "Acid Jazz", "Acid Punk", "Acoustic", "Alternative", "Altern Rock", "Ambient", "Anime", "Avantgarde","Ballad", "Bass", "Beat", "Bebob", "Big Band", "Black Metal", "Bluegrass", "Blues", "Booty Bass", "Brit Pop", "Cabaret","Celtic", "Chamber Music", "Chanson", "Chorus", "Christian Gangsta Rap", "Christian Rap", "Christian Rock", "Classical","Classic Rock", "Club", "Club-House", "Comedy", "Contemporary Christian", "Country", "Crossover", "Cult", "Dance", "Dance Hall","Darkwave", "Death Metal", "Disco", "Dream", "Drum & Bass", "Drum Solo", "Duet", "Easy Listening", "Electronic", "Ethnic","Eurodance", "Euro-House", "Euro-Techno", "Fast Fusion", "Folk", "Folklore", "Folk/Rock", "Freestyle", "Funk", "Fusion","Game", "Gangsta", "Goa", "Gospel", "Gothic", "Gothic Rock", "Grunge",  "Hardcore", "Hard Rock", "Heavy Metal", "Hip-Hop","House", "Humour", "Indie", "Industrial", "Instrumental", "Instrumental Pop", "Instrumental Rock", "Jazz", "Jazz+Funk","JPop", "Jungle", "Latin", "Lo-Fi", "Meditative", "Merengue", "Metal", "Musical", "National Folk", "Native American","Neger punk", "New Age", "New Wave", "Noise", "Oldies", "Opera", "Other", "Polka", "Polsk Punk", "Pop", "Pop-Folk", "Pop/Funk","Porn Groove", "Power Ballad", "Pranks", "Primus", "Progressive Rock", "Psychadelic", "Psychedelic Rock", "Punk", "Punk Rock","Rap", "Rave", "R&B", "Reggae", "Retro", "Revival", "Rhythmic Soul", "Rock", "Rock & Roll", "Salsa", "Samba", "Satire","Showtunes", "Ska", "Slow Jam", "Slow Rock", "Sonata", "Soul", "Sound Clip", "Soundtrack", "Southern Rock", "Space", "Speech","Swing", "Symphonic Rock", "Symphony", "Synthpop", "Tango", "Techno",  "Techno-Industrial", "Terror", "Thrash Metal", "Top 40","Trailer", "Trance", "Tribal", "Trip-Hop", "Vocal", 0
};
static QString id3_genres[] =
{
    "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop", "Jazz", "Metal",
    "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae", "Rock", "Techno", "Industrial",
    "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk",
    "Fusion", "Trance", "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise",
    "Altern Rock", "Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic",
    "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta",
    "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave", "Showtunes",
    "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock",
    "Folk", "Folk/Rock", "National Folk", "Fast Fusion", "Swing", "Bebob", "Latin", "Revival", "Celtic", "Bluegrass",
    "Avantgarde", "Gothic Rock", "Progressive Rock", "Psychedelic Rock", "Symphonic Rock",
    "Slow Rock", "Big Band", "Chorus", "Easy Listening", "Acoustic",
    "Humour", "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove",
    "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle",
    "Duet", "Punk Rock", "Drum Solo", "A Capella", "Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club-House", "Hardcore",
    "Terror", "Indie", "BritPop", "Negerpunk", "Polsk Punk", "Beat", "Christian Gangsta Rap", "Heavy Metal", "Black Metal",
    "Crossover", "Contemporary Christian", "Christian Rock", "Merengue", "Salsa", "Thrash Metal", "Anime", "JPop", "Synthpop"
};
//---//

#endif
